@@ -2,16 +2,20 @@ use async_trait::async_trait;
 use serde::de::DeserializeOwned;
 use serde::Serialize;
 
-use crate::{AggregateError, DomainEvent};
+use crate::DomainEvent;
 
 /// In CQRS (and Domain Driven Design) an `Aggregate` is the fundamental component that
 /// encapsulates the state and application logic (aka business rules) for the application.
-/// An `Aggregate` is always an entity along with all objects associated with it.
+/// An `Aggregate` is always composed of a
+/// [DDD entity](https://docs.microsoft.com/en-us/dotnet/architecture/microservices/microservice-ddd-cqrs-patterns/microservice-domain-model#the-domain-entity-pattern)
+/// along with all entities and
+/// [value objects](https://docs.microsoft.com/en-us/dotnet/architecture/microservices/microservice-ddd-cqrs-patterns/microservice-domain-model#the-value-object-pattern)
+/// associated with it.
 ///
-/// # Examples
+/// # Example of a 'Customer' aggregate
 /// ```rust
-/// # use cqrs_es::doc::{CustomerEvent, CustomerCommand};
-/// # use cqrs_es::{Aggregate, AggregateError, UserErrorPayload};
+/// # use cqrs_es::doc::{CustomerEvent, CustomerError, CustomerCommand, CustomerService};
+/// # use cqrs_es::{Aggregate, AggregateError};
 /// # use serde::{Serialize,Deserialize};
 /// # use async_trait::async_trait;
 /// #[derive(Default,Serialize,Deserialize)]
@@ -24,11 +28,13 @@ use crate::{AggregateError, DomainEvent};
 /// impl Aggregate for Customer {
 ///     type Command = CustomerCommand;
 ///     type Event = CustomerEvent;
-///     type Error = UserErrorPayload;
+///     type Error = CustomerError;
+///     type Services = CustomerService;
+///
 ///
 ///     fn aggregate_type() -> String { "customer".to_string() }
 ///
-///     async fn handle(&self, command: Self::Command) -> Result<Vec<Self::Event>, AggregateError<UserErrorPayload>> {
+///     async fn handle(&self, command: Self::Command, service: &Self::Services) -> Result<Vec<Self::Event>, Self::Error> {
 ///         match command {
 ///             CustomerCommand::AddCustomerName{name: changed_name} => {
 ///                 if self.name.is_some() {
@@ -65,19 +71,22 @@ pub trait Aggregate: Default + Serialize + DeserializeOwned + Sync + Send {
     /// The error returned when a command fails due to business logic.
     /// This is used to provide feedback to the user as to the nature of why the command was refused.
     type Error: std::error::Error;
+    /// The external services required for the logic within the Aggregate
+    type Services: Send + Sync;
     /// The aggregate type is used as the unique identifier for this aggregate and its events.
-    /// This is used for persisting the events or aggregate snapshot to a database.
+    /// This is used for persisting the events and snapshots to a database.
     fn aggregate_type() -> String;
     /// This method consumes and processes commands.
     /// The result should be either a vector of events if the command is successful,
     /// or an error if the command is rejected.
     ///
-    /// *All business logic should be placed here*.
+    /// _All business logic belongs in this method_.
     ///
     /// ```rust
-    /// use cqrs_es::{Aggregate, AggregateError, UserErrorPayload};
+    /// # use std::sync::Arc;
+    /// use cqrs_es::{Aggregate, AggregateError};
     /// # use serde::{Serialize, Deserialize, de::DeserializeOwned};
-    /// # use cqrs_es::doc::{CustomerCommand, CustomerEvent};
+    /// # use cqrs_es::doc::{CustomerCommand, CustomerError, CustomerEvent, CustomerService};
     /// # use async_trait::async_trait;
     /// #[derive(Default,Serialize,Deserialize)]
     /// # struct Customer {
@@ -88,9 +97,10 @@ pub trait Aggregate: Default + Serialize + DeserializeOwned + Sync + Send {
     /// # impl Aggregate for Customer {
     /// #     type Command = CustomerCommand;
     /// #     type Event = CustomerEvent;
-    /// #     type Error = UserErrorPayload;
+    /// #     type Error = CustomerError;
+    /// #     type Services = CustomerService;
     /// #     fn aggregate_type() -> String { "customer".to_string() }
-    /// async fn handle(&self, command: CustomerCommand) -> Result<Vec<CustomerEvent>, AggregateError<UserErrorPayload>> {
+    /// async fn handle(&self, command: Self::Command, service: &Self::Services) -> Result<Vec<Self::Event>, Self::Error> {
     ///     match command {
     ///         CustomerCommand::AddCustomerName{name: changed_name} => {
     ///             if self.name.is_some() {
@@ -110,23 +120,26 @@ pub trait Aggregate: Default + Serialize + DeserializeOwned + Sync + Send {
     async fn handle(
         &self,
         command: Self::Command,
-    ) -> Result<Vec<Self::Event>, AggregateError<Self::Error>>;
+        service: &Self::Services,
+    ) -> Result<Vec<Self::Event>, Self::Error>;
     /// This is used to update the aggregate's state once an event has been committed.
-    /// Events emitted from the `handle` method will be applied using this method
+    /// Any events returned from the `handle` method will be applied using this method
     /// in order to populate the state of the aggregate instance.
     ///
-    /// When these events are applied varies with the source of truth used:
-    /// - event sourced - Events are applied every time the aggregate is loaded.
+    /// The source of truth used in the CQRS framework determines when the events are
+    /// applied to an aggregate:
+    /// - event sourced - All events are applied every time the aggregate is loaded.
     /// - aggregate sourced - Events are applied immediately after they are returned from `handle`
     /// (and before they are committed) and the resulting aggregate instance is serialized and persisted.
     /// - snapshots - Uses a combination of the above patterns.
     ///
-    /// *No business logic should be placed here*, this is only used for updating the aggregate state.
+    /// _No business logic should be placed here_, this is only used for updating the aggregate state.
     ///
     /// ```rust
+    /// # use std::sync::Arc;
     /// # use serde::{Serialize, Deserialize, de::DeserializeOwned};
-    /// # use cqrs_es::doc::{CustomerCommand, CustomerEvent};
-    /// use cqrs_es::{Aggregate, AggregateError, UserErrorPayload};
+    /// # use cqrs_es::doc::{CustomerCommand, CustomerError, CustomerEvent, CustomerService};
+    /// use cqrs_es::{Aggregate, AggregateError};
     /// use async_trait::async_trait;
     /// #[derive(Default,Serialize,Deserialize)]
     /// # struct Customer {
@@ -137,9 +150,10 @@ pub trait Aggregate: Default + Serialize + DeserializeOwned + Sync + Send {
     /// # impl Aggregate for Customer {
     /// #     type Command = CustomerCommand;
     /// #     type Event = CustomerEvent;
-    /// #     type Error = UserErrorPayload;
+    /// #     type Error = CustomerError;
+    /// #     type Services = CustomerService;
     /// #     fn aggregate_type() -> String { "customer".to_string() }
-    /// # async fn handle(&self, command: CustomerCommand) -> Result<Vec<CustomerEvent>, AggregateError<UserErrorPayload>> {
+    /// # async fn handle(&self, command: Self::Command, service: &Self::Services) -> Result<Vec<Self::Event>, Self::Error> {
     /// # Ok(vec![])
     /// # }
     /// fn apply(&mut self, event: Self::Event) {
