@@ -6,7 +6,7 @@ use std::fmt::{Display, Formatter};
 use crate::persist::{
     PersistedEventRepository, PersistenceError, SerializedEvent, SerializedSnapshot,
 };
-use crate::{Aggregate, AggregateError, DomainEvent};
+use crate::{Aggregate, DomainEvent};
 
 #[derive(Debug, Serialize, Deserialize, PartialEq, Clone)]
 pub enum MyEvents {
@@ -35,6 +35,7 @@ impl Aggregate for MyAggregate {
     type Command = MyCommands;
     type Event = MyEvents;
     type Error = MyUserError;
+    type Services = MyService;
 
     fn aggregate_type() -> String {
         "MyAggregate".to_string()
@@ -43,12 +44,11 @@ impl Aggregate for MyAggregate {
     async fn handle(
         &self,
         command: Self::Command,
-    ) -> Result<Vec<Self::Event>, AggregateError<Self::Error>> {
+        _service: &Self::Services,
+    ) -> Result<Vec<Self::Event>, Self::Error> {
         match command {
             MyCommands::DoSomething => Ok(vec![MyEvents::SomethingWasDone]),
-            MyCommands::BadCommand => Err(AggregateError::UserError(
-                "the expected error message".into(),
-            )),
+            MyCommands::BadCommand => Err("the expected error message".into()),
         }
     }
 
@@ -79,11 +79,15 @@ impl From<&str> for MyUserError {
     }
 }
 
+#[derive(Debug, Clone, Default)]
+pub struct MyService;
+
 #[async_trait]
 impl Aggregate for Customer {
     type Command = CustomerCommand;
     type Event = CustomerEvent;
-    type Error = MyUserError;
+    type Error = CustomerError;
+    type Services = CustomerService;
 
     fn aggregate_type() -> String {
         "Customer".to_string()
@@ -92,13 +96,12 @@ impl Aggregate for Customer {
     async fn handle(
         &self,
         command: Self::Command,
-    ) -> Result<Vec<Self::Event>, AggregateError<Self::Error>> {
+        _service: &Self::Services,
+    ) -> Result<Vec<Self::Event>, Self::Error> {
         match command {
             CustomerCommand::AddCustomerName { name: changed_name } => {
                 if self.name.as_str() != "" {
-                    return Err(AggregateError::UserError(
-                        "a name has already been added for this customer".into(),
-                    ));
+                    return Err("a name has already been added for this customer".into());
                 }
                 Ok(vec![CustomerEvent::NameAdded { name: changed_name }])
             }
@@ -130,6 +133,26 @@ impl Default for Customer {
     }
 }
 
+#[derive(Clone, Debug, Serialize, Deserialize, PartialEq)]
+pub struct CustomerError(String);
+
+impl From<&str> for CustomerError {
+    fn from(message: &str) -> Self {
+        CustomerError(message.to_string())
+    }
+}
+
+impl Display for CustomerError {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        write!(f, "{}", self.0)
+    }
+}
+
+impl std::error::Error for CustomerError {}
+
+#[derive(Clone, Default)]
+pub struct CustomerService;
+
 #[derive(Clone, Debug, Serialize, Deserialize, PartialEq)]
 pub enum CustomerEvent {
     NameAdded { name: String },
@@ -165,7 +188,7 @@ mod doc_tests {
 
     #[test]
     fn test_add_name() {
-        CustomerTestFramework::default()
+        CustomerTestFramework::with(CustomerService::default())
             .given_no_previous_events()
             .when(CustomerCommand::AddCustomerName {
                 name: "John Doe".to_string(),
@@ -177,7 +200,7 @@ mod doc_tests {
 
     #[test]
     fn test_add_name_again() {
-        CustomerTestFramework::default()
+        CustomerTestFramework::with(CustomerService::default())
             .given(vec![CustomerEvent::NameAdded {
                 name: "John Doe".to_string(),
             }])
