@@ -32,11 +32,15 @@ impl From<&str> for TestError {
     }
 }
 
+#[derive(Clone, Debug)]
+pub struct TestService;
+
 #[async_trait]
 impl Aggregate for TestAggregate {
     type Command = TestCommand;
     type Event = TestEvent;
     type Error = TestError;
+    type Services = TestService;
 
     fn aggregate_type() -> String {
         "TestAggregate".to_string()
@@ -44,8 +48,9 @@ impl Aggregate for TestAggregate {
 
     async fn handle(
         &self,
-        command: TestCommand,
-    ) -> Result<Vec<TestEvent>, AggregateError<Self::Error>> {
+        command: Self::Command,
+        _service: &Self::Services,
+    ) -> Result<Vec<TestEvent>, Self::Error> {
         match &command {
             TestCommand::CreateTest(command) => {
                 let event = TestEvent::Created(Created {
@@ -57,7 +62,7 @@ impl Aggregate for TestAggregate {
             TestCommand::ConfirmTest(command) => {
                 for test in &self.tests {
                     if test == &command.test_name {
-                        return Err(AggregateError::UserError("test already performed".into()));
+                        return Err("test already performed".into());
                     }
                 }
                 let event = TestEvent::Tested(Tested {
@@ -237,9 +242,8 @@ type ThisTestFramework = TestFramework<TestAggregate>;
 #[test]
 fn test_framework_test() {
     let test_name = "test A";
-    let test_framework = ThisTestFramework::default();
 
-    test_framework
+    ThisTestFramework::with(TestService)
         .given(vec![TestEvent::Created(Created {
             id: "test_id_A".to_string(),
         })])
@@ -250,7 +254,7 @@ fn test_framework_test() {
             test_name: test_name.to_string(),
         })]);
 
-    test_framework
+    ThisTestFramework::with(TestService)
         .given(vec![TestEvent::Tested(Tested {
             test_name: test_name.to_string(),
         })])
@@ -264,9 +268,8 @@ fn test_framework_test() {
 #[should_panic]
 fn test_framework_failure_test() {
     let test_name = "test A";
-    let test_framework = ThisTestFramework::default();
 
-    test_framework
+    ThisTestFramework::with(TestService)
         .given(vec![TestEvent::Tested(Tested {
             test_name: test_name.to_string(),
         })])
@@ -282,9 +285,8 @@ fn test_framework_failure_test() {
 #[should_panic]
 fn test_framework_failure_test_b() {
     let test_name = "test A";
-    let test_framework = ThisTestFramework::default();
 
-    test_framework
+    ThisTestFramework::with(TestService)
         .given(vec![TestEvent::Created(Created {
             id: "test_id_A".to_string(),
         })])
@@ -302,7 +304,7 @@ async fn framework_test() {
     let delivered_events = Default::default();
     let view = TestView::new(Arc::clone(&delivered_events));
 
-    let cqrs = CqrsFramework::new(event_store, vec![Box::new(view)]);
+    let cqrs = CqrsFramework::new(event_store, vec![Box::new(view)], TestService);
     let uuid = uuid::Uuid::new_v4().to_string();
     let id = uuid.clone();
     let metadata = metadata();
